@@ -35,6 +35,7 @@ our @EXPORT = qw(
 );
 
 use Exporter qw(import);
+use Config;
 
 use Dpkg::ErrorHandling;
 use Dpkg::Gettext;
@@ -47,7 +48,7 @@ Dpkg::Compression - simple database of available compression methods
 
 =head1 DESCRIPTION
 
-This modules provides a few public funcions and a public regex to
+This modules provides a few public functions and a public regex to
 interact with the set of supported compression methods.
 
 =cut
@@ -55,7 +56,7 @@ interact with the set of supported compression methods.
 my $COMP = {
     gzip => {
 	file_ext => 'gz',
-	comp_prog => [ 'gzip', '--no-name', '--rsyncable' ],
+	comp_prog => [ 'gzip', '--no-name' ],
 	decomp_prog => [ 'gunzip' ],
 	default_level => 9,
     },
@@ -79,6 +80,24 @@ my $COMP = {
     },
 };
 
+#
+# XXX: The gzip package in Debian at some point acquired a Debian-specific
+# --rsyncable option via a vendor patch. Which is not present in most of the
+# major distributions, dpkg downstream systems, nor gzip upstream, who have
+# stated they will most probably not accept it because people should be using
+# pigz instead.
+#
+# This option should have never been accepted in dpkg, ever. But removing it
+# now would probably cause demands for tarring and feathering. In addition
+# we cannot use the Dpkg::Vendor logic because that would cause circular
+# module dependencies. The whole affair is pretty disgusting really.
+#
+# Check the perl Config to discern Debian and hopefully derivatives too.
+#
+if ($Config{cf_by} eq 'Debian Project') {
+    push @{$COMP->{gzip}->{comp_prog}}, '--rsyncable';
+}
+
 # XXX: Backwards compatibility, stop exporting on VERSION 2.00.
 ## no critic (Variables::ProhibitPackageVars)
 our $default_compression = 'xz';
@@ -92,7 +111,7 @@ our $compression_re_file_ext = qr/(?:$regex)/;
 
 =over 4
 
-=item my @list = compression_get_list()
+=item @list = compression_get_list()
 
 Returns a list of supported compression methods (sorted alphabetically).
 
@@ -152,7 +171,7 @@ sub compression_guess_from_filename {
     return;
 }
 
-=item my $regex = compression_get_file_extension_regex()
+=item $regex = compression_get_file_extension_regex()
 
 Returns a regex that matches a file extension of a file compressed with
 one of the supported compression methods.
@@ -163,7 +182,7 @@ sub compression_get_file_extension_regex {
     return $compression_re_file_ext;
 }
 
-=item my $comp = compression_get_default()
+=item $comp = compression_get_default()
 
 Return the default compression method. It is "xz" unless
 C<compression_set_default> has been used to change it.
@@ -186,7 +205,7 @@ sub compression_set_default {
     $default_compression = $method;
 }
 
-=item my $level = compression_get_default_level()
+=item $level = compression_get_default_level()
 
 Return the default compression level used when compressing data. It's "9"
 for "gzip" and "bzip2", "6" for "xz" and "lzma", unless
@@ -246,10 +265,6 @@ Default compression level is not global any more, it is per compressor type.
 
 Mark the module as public.
 
-=head1 AUTHOR
-
-Raphaël Hertzog <hertzog@debian.org>.
-
 =cut
 
 1;
